@@ -1,4 +1,4 @@
-{-# LANGUAGE DeriveDataTypeable, DeriveGeneric, FlexibleContexts #-}
+{-# LANGUAGE DeriveDataTypeable, DeriveGeneric, FlexibleContexts, CPP #-}
 
 {-
 Copyright (C) 2006-2013 John MacFarlane <jgm@berkeley.edu>
@@ -54,6 +54,7 @@ module Text.Pandoc.Definition ( Pandoc(..)
                               , MathType(..)
                               , Citation(..)
                               , CitationMode(..)
+                              , pandocTypesVersion
                               ) where
 
 import Data.Generics (Data, Typeable)
@@ -66,7 +67,13 @@ import GHC.Generics (Generic, Rep (..))
 import Data.String
 import Data.Char (toLower)
 import Data.Monoid
+#if MIN_VERSION_base(4,8,0)
+import Control.DeepSeq
+#else
 import Control.DeepSeq.Generics
+#endif
+import Paths_pandoc_types (version)
+import Data.Version (Version)
 
 data Pandoc = Pandoc Meta [Block]
               deriving (Eq, Ord, Read, Show, Typeable, Data, Generic)
@@ -233,11 +240,12 @@ data Inline
     | Cite [Citation]  [Inline] -- ^ Citation (list of inlines)
     | Code Attr String      -- ^ Inline code (literal)
     | Space                 -- ^ Inter-word space
+    | SoftBreak             -- ^ Soft line break
     | LineBreak             -- ^ Hard line break
     | Math MathType String  -- ^ TeX math (literal)
     | RawInline Format String -- ^ Raw inline
-    | Link [Inline] Target  -- ^ Hyperlink: text (list of inlines), target
-    | Image [Inline] Target -- ^ Image:  alt text (list of inlines), target
+    | Link Attr [Inline] Target  -- ^ Hyperlink: alt text (list of inlines), target
+    | Image Attr [Inline] Target -- ^ Image:  alt text (list of inlines), target
     | Note [Block]          -- ^ Footnote or endnote
     | Span Attr [Inline]    -- ^ Generic inline container with attributes
     deriving (Show, Eq, Ord, Read, Typeable, Data, Generic)
@@ -260,7 +268,8 @@ data CitationMode = AuthorInText | SuppressAuthor | NormalCitation
 -- derive generic instances of FromJSON, ToJSON:
 
 jsonOpts :: Aeson.Options
-jsonOpts = Aeson.Options{ Aeson.fieldLabelModifier = id
+jsonOpts = Aeson.defaultOptions{
+                          Aeson.fieldLabelModifier = id
                         , Aeson.constructorTagModifier = id
                         , Aeson.allNullaryToStringTag = False
                         , Aeson.omitNothingFields = False
@@ -341,6 +350,21 @@ instance ToJSON Pandoc
   where toJSON = toJSON'
 
 -- Instances for deepseq
+#if MIN_VERSION_base(4,8,0)
+instance NFData MetaValue
+instance NFData Meta
+instance NFData Citation
+instance NFData Alignment
+instance NFData Inline
+instance NFData MathType
+instance NFData Format
+instance NFData CitationMode
+instance NFData QuoteType
+instance NFData ListNumberDelim
+instance NFData ListNumberStyle
+instance NFData Block
+instance NFData Pandoc
+#else
 instance NFData MetaValue where rnf = genericRnf
 instance NFData Meta where rnf = genericRnf
 instance NFData Citation where rnf = genericRnf
@@ -354,3 +378,7 @@ instance NFData ListNumberDelim where rnf = genericRnf
 instance NFData ListNumberStyle where rnf = genericRnf
 instance NFData Block where rnf = genericRnf
 instance NFData Pandoc where rnf = genericRnf
+#endif
+
+pandocTypesVersion :: Version
+pandocTypesVersion = version
